unit PreferencesFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, Menus;

type

  TPreferences = record
    Version: Single; // we also need to save program version, since users could copy the old config file over the new one

    ServerIP: string[255];
    ServerPort: string[255];
    TabStyle: Byte;
    TimeStamps: Boolean;

    Username: string[255];
    Password: string[255];
  end;

  TPreferencesForm = class(TForm)
    ApplyAndCloseButton: TButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    UsernameEdit: TEdit;
    PasswordEdit: TEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    Edit1: TEdit;
    Edit2: TEdit;
    TabSheet3: TTabSheet;
    RadioButton4: TRadioButton;
    RadioButton5: TRadioButton;
    RadioButton6: TRadioButton;
    Label6: TLabel;
    CheckBox1: TCheckBox;
    CancelAndCloseButton: TButton;
    SpeedButton1: TSpeedButton;
    AddressSpeedButton: TSpeedButton;
    AddressPopupMenu: TPopupMenu;
    Item1: TMenuItem;
    Item3: TMenuItem;
    Item2: TMenuItem;
    FontDialog1: TFontDialog;

    procedure UpdatePreferencesTo(var p: TPreferences);
    procedure UpdatePreferencesFrom(p: TPreferences);
    procedure SavePreferences(p: TPreferences; FileName: string);
    procedure LoadPreferences(var p: TPreferences; FileName: string);
    function GetConfigFileVersion(FileName: string): Single;

    procedure ApplyAndCloseButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CancelAndCloseButtonClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure AddressSpeedButtonClick(Sender: TObject);
    procedure Item1Click(Sender: TObject);
    procedure Item3Click(Sender: TObject);
    procedure Item2Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  DEFAULT_PREFERENCES: TPreferences = (ServerIP:'taspringmaster.clan-sy.com'; ServerPort:'8200'; TabStyle:2; TimeStamps: True; Username:''; Password:'');
  PREFERENCES_FILENAME = 'config.dat';

var
  PreferencesForm: TPreferencesForm;
  Preferences: TPreferences;
  SaveOnClose: Boolean;

implementation

uses Unit1, Misc;

{$R *.dfm}

procedure TPreferencesForm.ApplyAndCloseButtonClick(Sender: TObject);
begin
  if UsernameEdit.Text = '' then
  begin
    MessageDlg('"Username" field should not be left empty!', mtWarning, [mbOK], 0);
    Exit;
  end;

  SaveOnClose := True;
  Close;
end;

procedure TPreferencesForm.SavePreferences(p: TPreferences; FileName: string);
var
  f: file of TPreferences;
begin
  try
    p.Version := Misc.VersionStringToFloat(VERSION_NUMBER);
  except
  end;

  {$I-}
  AssignFile(f, FileName);
  Rewrite(f);
  Write(f, p);
  CloseFile(f);
  {$I+}
end;

procedure TPreferencesForm.LoadPreferences(var p: TPreferences; FileName: string);
var
  f: file of TPreferences;
begin
  if not FileExists(FileName) then Exit;

  if (GetConfigFileVersion(FileName) <> VersionStringToFloat(VERSION_NUMBER))
  or (Misc.GetFileSize(FileName) <> SizeOf(TPreferences)) then
  begin
    MessageDlg('The config file is outdated/corrupt. It will be now deleted and replaced with a new one on program exit.', mtWarning, [mbOK], 0);
    DeleteFile(FileName);
    Exit;
  end;

  {$I-}
  AssignFile(f, FileName);
  FileMode := 0;  {Set file access to read only }
  Reset(f);
  Read(f, p);
  CloseFile(f);
  {$I+}
end;

function TPreferencesForm.GetConfigFileVersion(FileName: string): Single;
var
  f: file of Single;
begin
  Result := 0;

  if not FileExists(FileName) then Exit;

  {$I-}
  AssignFile(f, FileName);
  FileMode := 0;  {Set file access to read only }
  Reset(f);
  Read(f, Result);
  CloseFile(f);
  {$I+}
end;

{ Saves radio buttons/check boxes/etc. states into variable p }
procedure TPreferencesForm.UpdatePreferencesTo(var p: TPreferences);
var
  i: Integer;
begin
  p.ServerIP := Edit1.Text;
  p.ServerPort := Edit2.Text;

  if RadioButton4.Checked then i := 0
  else if RadioButton5.Checked then i := 1
  else if RadioButton6.Checked then i := 2
  else i := 0; // this should not happen!

  p.TabStyle := i;

  p.TimeStamps := CheckBox1.Checked;

  p.Username := UsernameEdit.Text;
  p.Password := PasswordEdit.Text;
end;

{ applies preferences from variable p to VCL controls (radio buttons, check boxes, etc.) }
procedure TPreferencesForm.UpdatePreferencesFrom(p: TPreferences);
begin
  Edit1.Text := p.ServerIP;
  Edit2.Text := p.ServerPort;

  if p.TabStyle = 0 then RadioButton4.Checked := True
  else if p.TabStyle = 1 then RadioButton5.Checked := True
  else if p.TabStyle = 2 then RadioButton6.Checked := True
  else RadioButton4.Checked := True; // this should not happen!

  CheckBox1.Checked := p.TimeStamps;

  MainForm.PageControl1.Style := TTabStyle(p.TabStyle);

  UsernameEdit.Text := p.Username;
  PasswordEdit.Text := p.Password;
end;

procedure TPreferencesForm.FormCreate(Sender: TObject);
begin
  Preferences := DEFAULT_PREFERENCES;
  if not FileExists(PREFERENCES_FILENAME) then
  begin
    UpdatePreferencesFrom(Preferences);
    MainForm.AddLog('Welcome to the new TA Spring Client, version 0.10.', Colors.MOTD);
    MainForm.AddLog('', Colors.MOTD);
    MainForm.AddLog('To get started, click on the Connect icon in the top', Colors.MOTD);
    MainForm.AddLog('left corner.', Colors.MOTD);
    MainForm.AddLog('', Colors.MOTD);
    MainForm.AddLog('The new lobby uses a brand new server as well, which', Colors.MOTD);
    MainForm.AddLog('should perform better than the previous one. The ', Colors.MOTD);
    MainForm.AddLog('downside is that you will have register your', Colors.MOTD);
    MainForm.AddLog('username again. Sorry for the inconvenience.', Colors.MOTD);
    MainForm.AddLog('Hopefully this is the last server replacement we', Colors.MOTD);
    MainForm.AddLog('will have to do though.', Colors.MOTD);
  end
  else
  begin
    LoadPreferences(Preferences, PREFERENCES_FILENAME);
    UpdatePreferencesFrom(Preferences);
  end;

  PageControl1.ActivePageIndex := 0;
end;

procedure TPreferencesForm.FormDestroy(Sender: TObject);
begin
  SavePreferences(Preferences, PREFERENCES_FILENAME);
end;

procedure TPreferencesForm.FormShow(Sender: TObject);
begin
  SaveOnClose := False;
end;

procedure TPreferencesForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ActiveControl := nil;
  { Wonder why we need ActiveControl := nil ? We need it in this case: if focus was on the radio button
    when we closed the form (by clicking the 'X' so that focus remained on radio button), then that same
    radio button would get automatically selected when form is shown and so ignoring any changes we made
    in OnShow event to Checked properties of any of the radio buttons. If you call TRadioButton.SetFocus,
    it will automatically set it's Checked property to true. }

  if SaveOnClose then
  begin
    UpdatePreferencesTo(Preferences); // save changes to p
    UpdatePreferencesFrom(Preferences); // update changes from p
  end
  else
  begin
    UpdatePreferencesFrom(Preferences);
  end;
end;

procedure TPreferencesForm.CancelAndCloseButtonClick(Sender: TObject);
begin
  SaveOnClose := False;
  Close;
end;

procedure TPreferencesForm.SpeedButton1Click(Sender: TObject);
begin
  if Status.ConnectionState = Connected then
  begin
    MessageDlg('You are not allowed to register new account while being logged-in!', mtError, [mbOK], 0);
    Exit;
  end;

  if (UsernameEdit.Text = '') or (PasswordEdit.Text = '') then
  begin
    MessageDlg('Username/password should not be blank!', mtError, [mbOK], 0);
    Exit;
  end;

  PostMessage(MainForm.Handle, WM_DOREGISTER, 0, 0);
  ApplyAndCloseButton.OnClick(nil);
end;

procedure TPreferencesForm.AddressSpeedButtonClick(Sender: TObject);
var
  p: TPoint;
begin
  with AddressSpeedButton do
    p := AddressSpeedButton.ClientToScreen(Point(0, 0));

  AddressPopupMenu.Popup(p.x, p.y);
end;

procedure TPreferencesForm.Item1Click(Sender: TObject);
begin
  Edit1.Text := 'taspringmaster.clan-sy.com';
end;

procedure TPreferencesForm.Item3Click(Sender: TObject);
begin
  Edit1.Text := 'localhost';
end;

procedure TPreferencesForm.Item2Click(Sender: TObject);
begin
  Edit1.Text := 'cheetah.sentvid.org';
end;

end.
