using System;
using System.Collections;
using System.Windows.Forms;
using ClientControls;

using System.IO;

namespace SpringClient
{
	/// <summary>
	/// Summary description for BattleStateManager.
	/// </summary>
	public class BattleStateManager
	{
		private Hashtable		m_Members;
		private TeamInfo[]		m_Teams;
		private AllyTeamInfo[]	m_AllyTeams;
		private ArrayList		m_OrderedMembers;

		private BattleStateManager.BattleMemberInfo m_Self;
		private const int		MAXTEAMS = 7;

		private StreamWriter			m_DebugWriter;
		private bool			m_Debug = false;

		public int metal;
		public int energy;
		public int maxUnits;

		public int startPosType;
		public int gameGoalType;

		public Hashtable Members
		{
			get
			{
				return m_Members;
			}
		}

		public TeamInfo[] Teams
		{
			get
			{
				return m_Teams;
			}
		}

		public AllyTeamInfo[] AllyTeams
		{
			get
			{
				return m_AllyTeams;
			}
		}

		public ArrayList OrderedMembers
		{
			get
			{
				return m_OrderedMembers;
			}
		}

		public enum TeamColor
		{
			BLUE = 0,
			RED = 1,
			GREEN = 2,
			YELLOW = 3,
			ORANGE = 4,
			WHITE = 5,
			BLACK = 6,
			NONE = 0xFF
		}

		public enum SIDE
		{
			ARM = 0,
			CORE = 1
		}

		public class BattleMemberInfo
		{
			public int			m_ID;
			public int			m_ConnID;
			public bool			m_Synced = false;
			public string		m_Name;
			public string		m_Clan;
			public byte			m_InternalID;
			public bool			m_bSpectator;
			public TeamColor	m_TeamID;
			public ListViewItem	m_lvi;
			public bool			m_currUser = false;
			public ComboBox		m_CBTeam;
			public ComboBox		m_CBAllyTeam;
			public CheckBox		m_checkReady;
			public AllyState	m_allyState;
		}

		public class TeamInfo
		{
			public byte			m_TeamID;
			public byte			m_AllyTeamID;
			public byte			m_InternalID;
			public ArrayList	m_Members;
			public SIDE			m_Side;
		}

		public class AllyTeamInfo
		{
			public byte			m_AllyTeamID;
			public ArrayList	m_Members;
			public ArrayList	m_AlliedTo;
		}

		private ClientControls.ListViewEx m_lv;
		private Button m_Start;
		private MenuItem m_menuOptions;

		public BattleStateManager(ClientControls.ListViewEx lve, Button Start, MenuItem menuOptions)
		{
			//
			// TODO: Add constructor logic here
			//

			if ( m_Debug )
			{
				m_DebugWriter = new StreamWriter(File.OpenWrite("bsmlog.txt"));
			}
			m_Start = Start;
			m_lv = lve;
			m_menuOptions = menuOptions;
			m_Members = new Hashtable();
			m_Teams = new TeamInfo[MAXTEAMS];
			m_AllyTeams = new AllyTeamInfo[MAXTEAMS];
			m_OrderedMembers = new ArrayList();
			for ( byte i = 0; i < MAXTEAMS; i++ )
			{
				m_Teams[i] = new TeamInfo();
				m_AllyTeams[i] = new AllyTeamInfo();
				m_Teams[i].m_TeamID = i;
				m_Teams[i].m_AllyTeamID = 0;
				m_AllyTeams[i].m_AllyTeamID = i;
				m_Teams[i].m_Members = new ArrayList();
				m_AllyTeams[i].m_Members = new ArrayList();
		//		m_AllyTeams[i].m_Members.Add(m_Teams[i]);
				m_AllyTeams[i].m_AlliedTo = new ArrayList();
				m_Teams[i].m_InternalID = 0;
			}
			metal=1000;
			energy=1000;
			maxUnits=500;
			startPosType=0;
			gameGoalType=0;
		}

		~BattleStateManager()
		{
			if ( m_Debug )
			{
				m_DebugWriter.Close();
			}
		}

		public bool SetSync(int id)
		{
			try
			{
				BattleStateManager.BattleMemberInfo bmi = (BattleStateManager.BattleMemberInfo)m_Members[id];
				bmi.m_Synced = true;
				bmi.m_lvi.SubItems[5].Text = "*";
				return true;
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				return false;
			}
				/*
			catch ( ArgumentOutOfRangeException )
			{
				// fatal battle-room killing error.  will handle this later
				return false;
			}
			catch ( NullReferenceException )
			{
				return false;
			}*/
		}

		public bool SetUnsync(int id)
		{
			try
			{
				BattleStateManager.BattleMemberInfo bmi = (BattleStateManager.BattleMemberInfo)m_Members[id];
				bmi.m_Synced = false;
				bmi.m_lvi.SubItems[5].Text = "";
				return true;
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				return false;
			}
			/*
			catch ( ArgumentOutOfRangeException )
			{
				// fatal battle-room killing error.  will handle this later
				return false;
			}
			catch ( NullReferenceException )
			{
				return false;
			}*/
		}

		public void SetUnsyncAll()
		{
			try
			{
				IDictionaryEnumerator ide = m_Members.GetEnumerator();
				while ( ide.MoveNext() )
				{
					 BattleStateManager.BattleMemberInfo bmi = (BattleStateManager.BattleMemberInfo)ide.Value;
					bmi.m_Synced = false;
					bmi.m_lvi.SubItems[5].Text = "";
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
				/*
			catch ( InvalidOperationException )
			{
			}
			catch ( NullReferenceException )
			{
			}*/
		}

		// sets the user denoted by id to the leader of the Team
		public void SetTeamLeader(int id)
		{
	//		try
			{
				if ( !m_Members.ContainsKey(id) )
					throw new Exception(string.Format("User {0} does not exist in the battle room", id));
				BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id];
				// do nothing for spectator
				if ( bmi.m_bSpectator )
					return;
				TeamInfo ti = m_Teams[(byte)bmi.m_TeamID];
				foreach ( BattleMemberInfo bmi2 in ti.m_Members )
				{
					// essentially, bubble up all users that came before the new leader
					if ( object.ReferenceEquals(bmi, bmi2) )
						continue;
					if ( bmi2.m_InternalID == 0 )
					{
						bmi2.m_CBAllyTeam.Enabled = false;
						if ( object.Equals(bmi2, m_Self) )
						{
							m_menuOptions.Enabled = false;
						}
					}
					if ( bmi2.m_InternalID < bmi.m_InternalID )
						bmi2.m_InternalID++;
				}
				// and set the leaders id to the "leader" position
				bmi.m_InternalID = 0;
				// now set this user as the "leader" by enabling the ally team box
				if ( object.Equals(bmi, m_Self) )
				{
					bmi.m_CBAllyTeam.Enabled = true;
					m_menuOptions.Enabled = true;
				}
			}
				/*
			catch ( Exception ex )
			{
				throw ex;
			}*/
		}

		public void SetReady( int id )
		{
			try
			{
				if ( !m_Members.Contains(id) )
					return;
				BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id];
				if ( bmi.m_bSpectator )
					return;
				bmi.m_checkReady.Checked = true;
				if ( m_Start != null )
				{
					IDictionaryEnumerator ide = m_Members.GetEnumerator();
					bool alltrue = true;
					while ( ide.MoveNext() )
					{
						if ( ((BattleMemberInfo)ide.Value).m_bSpectator )
							continue;
						if ( !((BattleMemberInfo)ide.Value).m_checkReady.Checked)
						{
							alltrue = false;
							break;
						}
					}
					if ( alltrue )
					{
						m_Start.Enabled = true;
					}
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void SetNotReady ( int id )
		{
			try
			{
				if ( !m_Members.Contains(id) )
					return;
				BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id];
				if ( bmi.m_bSpectator )
					return;
				bmi.m_checkReady.Checked = false;
				if ( m_Start != null )
				{
					m_Start.Enabled = false;
				}
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		public void Clear(bool server)
		{
			m_Members.Clear();
			m_OrderedMembers.Clear();
			foreach ( AllyTeamInfo ati in m_AllyTeams )
			{
				ati.m_Members.Clear();
				ati.m_AlliedTo.Clear();
			}
			foreach( TeamInfo ti in m_Teams)
			{
				ti.m_Members.Clear();
				ti.m_InternalID = 0;
				ti.m_AllyTeamID = 0xFF;
				ti.m_Side = SIDE.ARM; // arm by default
	//			if ( server )
	//				m_AllyTeams[0].m_Members.Add(ti.m_TeamID);
			}
		}

		public TeamColor GetUserTeam(int id)
		{
			if ( !m_Members.Contains(id) )
				throw new Exception(string.Format("User {0} not found", id));
			BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id];
			return bmi.m_TeamID;
		}

		public byte GetAllyTeam(TeamColor id)
		{
			return m_Teams[(byte)id].m_AllyTeamID;
		}

		public BattleMemberInfo AddMember(bool spectator, int id, bool self, TeamColor teamid, int internalTeamId, string name, string clan, byte ready)
		{
			BattleMemberInfo bmi = null;
	//		try
			{
				bmi = new BattleMemberInfo();
//				lock ( this )
				{
					if ( self )
						m_Self = bmi;
					if ( m_Start != null )
					{
						m_Start.Enabled = false;
					}
					bmi.m_ConnID = id;
					bmi.m_bSpectator = spectator;
					bmi.m_ID = id;
					bmi.m_TeamID = TeamColor.NONE;
					bmi.m_currUser = self;
					if ( internalTeamId == -1 )
						bmi.m_InternalID = (byte)m_Teams[(int)teamid].m_Members.Count;
					else
						bmi.m_InternalID = (byte)internalTeamId;
					bmi.m_lvi = new ListViewItem(new string[] {name,"","","","",""}, 2);
					m_lv.Items.Add(bmi.m_lvi);
					bmi.m_Name = name;
					bmi.m_Clan = clan;
					m_Members.Add(id, bmi);
					m_OrderedMembers.Add(bmi);

					if ( !spectator )
					{
						//		m_Teams[(int)teamid].m_Members.Add(bmi);
						bmi.m_lvi.ImageIndex = (int)m_Teams[(int)teamid].m_Side;
						// bmi.m_lvi.SubItems[0].Text = name;

						// add combo box which is always disabled for team selection.  leave it up to the
						// calling code to enable it (only if the member added is the one who is logged in)
						ComboBox cbTeamSelect = new ComboBox();
						CheckBox checkReady = new CheckBox();
						ComboBox cbAllyTeamSelect = new ComboBox();

						cbTeamSelect.DropDownStyle = ComboBoxStyle.DropDownList;

						for ( int i = 0; i < MAXTEAMS; i++ )
							cbTeamSelect.Items.Add(i.ToString());
						//			cbTeamSelect.SelectedIndex = (int)teamid;
						cbTeamSelect.Name = "TeamSelect";

						for ( int i = 0; i < MAXTEAMS; i++ )
							cbAllyTeamSelect.Items.Add(i.ToString());
						// cbAllyTeamSelect.SelectedIndex = (int)m_Teams[(byte)teamid].m_AllyTeamID;
						cbAllyTeamSelect.Name = "AllyTeamSelect";

						bmi.m_CBTeam = cbTeamSelect;
						bmi.m_CBAllyTeam = cbAllyTeamSelect;
						bmi.m_checkReady = checkReady;

						MemberChangeTeam(id, (byte)teamid, bmi.m_InternalID);

						if ( self )
						{
							cbTeamSelect.Enabled = true;
							cbTeamSelect.SelectedIndexChanged += new EventHandler(SelfTeamSelectedIndexChanged);
						}
						else
						{
							cbTeamSelect.Enabled = false;
							//			cbTeamSelect.SelectedIndexChanged += new EventHandler(OtherTeamSelectedIndexChanged);
						}
			
						m_lv.AddEmbeddedControl(cbTeamSelect, 1, m_lv.Items.Count-1, DockStyle.Fill);

					
						if ( self )
						{
							cbAllyTeamSelect.SelectedIndexChanged +=new EventHandler(SelfAllyTeamSelectedIndexChanged);
						}
						if ( self && bmi.m_InternalID == 0 )
						{
							cbAllyTeamSelect.Enabled = true;
						}
						else
						{
							cbAllyTeamSelect.Enabled = false;
							//						cbAllyTeamSelect.SelectedIndexChanged += new EventHandler(SelfAllyTeamSelectedIndexChanged);
						}

						cbAllyTeamSelect.DropDownStyle = ComboBoxStyle.DropDownList;

					
						checkReady.Checked = false;
						if ( ready != 0)
							checkReady.Checked = true;
						if ( self )
						{
							checkReady.Enabled = true;
							checkReady.CheckedChanged += new EventHandler(checkReady_CheckedChanged);
						}
						else
							checkReady.Enabled = false;

						//					bmi.m_allyState = new ClientControls.AllyState();
						/*
						if ( self )
						{
							bmi.m_allyState.Visible = false;
							bmi.m_allyState.Enabled = false;
						}
						*/
						//					bmi.m_allyState.Click += new EventHandler(m_allyState_Click);
						m_lv.AddEmbeddedControl(cbAllyTeamSelect, 2, m_lv.Items.Count-1, DockStyle.Fill);
						//					m_lv.AddEmbeddedControl(bmi.m_allyState, 3, m_lv.Items.Count-1, DockStyle.Fill);
						m_lv.AddEmbeddedControl(checkReady, 4, m_lv.Items.Count-1, DockStyle.Left);
					
						if ( self && bmi.m_InternalID == 0 )
							m_menuOptions.Enabled = true;
						else
							m_menuOptions.Enabled = false;
						UpdateAllyStates();
					}
					else
					{

					}

					DebugOut("AddMember", (byte)bmi.m_ConnID, 0);
				}
			}
			CheckReady();
			return bmi;
		}

		private void DebugOut(string func, byte from, byte to)
		{
			if ( !m_Debug )
				return;
			m_DebugWriter.WriteLine("{0} called with {1} to {2}", func, from.ToString(), to.ToString());
			m_DebugWriter.WriteLine("Teams");
			for ( int i = 0; i < m_Teams.Length; i++ )
			{
				m_DebugWriter.WriteLine("Team: {0}    Allyteam: {1}", i.ToString(), m_Teams[i].m_AllyTeamID.ToString());
				for ( int j = 0; j < m_Teams[i].m_Members.Count; j++ )
				{
					m_DebugWriter.WriteLine("\tMember: {0} Team: {1}", ((BattleMemberInfo)m_Teams[i].m_Members[j]).m_ConnID.ToString(),
						((BattleMemberInfo)m_Teams[i].m_Members[j]).m_TeamID.ToString());
				}
				m_DebugWriter.WriteLine("");
			}
			m_DebugWriter.WriteLine("AllyTeams");
			for ( int i = 0; i < m_AllyTeams.Length; i++ )
			{
				m_DebugWriter.WriteLine("AllyTeam: {0}", i.ToString());
				for ( int j = 0; j < m_AllyTeams[i].m_Members.Count; j++ )
				{
					m_DebugWriter.WriteLine("\tMember: {0}  AllyTeam: {1}", ((TeamInfo)m_AllyTeams[i].m_Members[j]).m_TeamID.ToString(),
						((TeamInfo)m_AllyTeams[i].m_Members[j]).m_AllyTeamID);
				}
				m_DebugWriter.WriteLine("");
			}
			m_DebugWriter.WriteLine("----------------------------------");
			m_DebugWriter.Flush();
		}
		private void SelfTeamSelectedIndexChanged( object sender, EventArgs e )
		{
			// send a packet notifying the server of the change
			ComboBox from = (ComboBox)sender;
			if ( !from.Enabled )
				return;  // do nothing if this box is enabled (means you have control)
			// else send a team select change
			Global.battleRoom.ChangeTeam(Global.battleRoom.GetID(), (BattleStateManager.TeamColor)from.SelectedIndex);
		}

		private void SelfAllyTeamSelectedIndexChanged( object sender, EventArgs e)
		{
//			System.Diagnostics.Debug.WriteLine("Ally Team Changed start");
			// send a packet notifying the server of the change
			ComboBox from = (ComboBox)sender;
			if ( !from.Enabled )
				return;  // do nothing if this box is enabled (means you have control)
			// else send a team select change
//			System.Diagnostics.Debug.WriteLine("Ally Team Changed EXECUTE");
			Global.battleRoom.ChangeAllyTeam(true, (byte)GetUserTeam(Global.battleRoom.GetID()), (byte)from.SelectedIndex);
		}

		public void TeamChangeSide ( byte team, byte toSide )
		{
			if ( team < 0 || team >= m_Teams.Length )
				return;
			TeamInfo ti = m_Teams[team];
			ti.m_Side = (SIDE)toSide;
			IDictionaryEnumerator ide = m_Members.GetEnumerator();
			while ( ide.MoveNext() )
			{
				BattleMemberInfo bmi = (BattleMemberInfo)ide.Value;
				if ( (byte)bmi.m_TeamID == team )
					bmi.m_lvi.ImageIndex = (int)toSide;
			}
		}

		public void MemberChangeTeam( int userid, byte toteam, byte internalid )
		{
			{
				if ( !m_Members.Contains(userid) )
					return;
				BattleMemberInfo bmi = (BattleMemberInfo)m_Members[userid];
				
				if ( bmi.m_bSpectator )
					return;

				byte team = (byte)bmi.m_TeamID;
				if ( team != 0xFF )
					m_Teams[team].m_Members.RemoveAt(bmi.m_InternalID);
				bmi.m_TeamID = (TeamColor)toteam;
		//		bmi.m_InternalID = (byte)m_Teams[(byte)bmi.m_TeamID].m_Members.Count;
				if ( internalid == 0xFF )
				{
					bmi.m_InternalID = (byte)m_Teams[(byte)bmi.m_TeamID].m_Members.Count;
					m_Teams[(byte)bmi.m_TeamID].m_Members.Add(bmi);
				}
				else
				{
					if ( internalid > (byte)m_Teams[(byte)bmi.m_TeamID].m_Members.Count )
					{
						for ( byte j = (byte)m_Teams[(byte)bmi.m_TeamID].m_Members.Count; j < internalid; j++ )
						{
							m_Teams[(byte)bmi.m_TeamID].m_Members.Insert(j, null);
						}
					}
					bmi.m_InternalID = internalid;
					m_Teams[(byte)bmi.m_TeamID].m_Members.Add(bmi);
				}

				// if the team you changed from has no members, then clean it up
				if ( team != 0xFF && m_Teams[team].m_Members.Count == 0 )
				{
					// cleanup
					byte allyteam = m_Teams[team].m_AllyTeamID;
					m_AllyTeams[allyteam].m_Members.Remove(m_Teams[team]);
					m_Teams[team].m_InternalID = 0;
					m_Teams[team].m_AllyTeamID = 0xFF;
					m_Teams[team].m_Side = SIDE.ARM;
					if ( m_AllyTeams[allyteam].m_Members.Count == 0 )
					{
						// then cleanup this too
						m_AllyTeams[allyteam].m_AlliedTo.Clear();
						foreach ( AllyTeamInfo ati in m_AllyTeams )
						{
							if ( ati.m_AlliedTo.Contains(allyteam) )
								ati.m_AlliedTo.Remove(allyteam);
						}
					}
				}

				// if the allyteam is -1, then assign it to one automatically, with the same id as itself
				if ( m_Teams[(byte)bmi.m_TeamID].m_AllyTeamID == 0xFF )
				{
					TeamChangeAllyTeam((byte)bmi.m_TeamID, (byte)bmi.m_TeamID);
				}

				bmi.m_lvi.ImageIndex = (int)m_Teams[toteam].m_Side;

				ComboBox cbTeam = bmi.m_CBTeam;//(ComboBox)m_lv.GetEmbeddedControl(1, index);
				ComboBox cbAllyTeam = bmi.m_CBAllyTeam;//(ComboBox)m_lv.GetEmbeddedControl(2, index);
				cbTeam.SelectedIndex = toteam;

				cbAllyTeam.SelectedIndex = m_Teams[(byte)bmi.m_TeamID].m_AllyTeamID;

				UpdateAllyStates();
			}
			DebugOut("MemberChangeTeam", (byte)userid, (byte)toteam);
		}

		private void UpdateAllyStates()
		{
			if ( m_Self == null )
				return;
			if ( m_Self.m_bSpectator )
				return;

			foreach ( AllyTeamInfo ati in m_AllyTeams )
			{
				byte tcount = 0;
				foreach ( TeamInfo ti in ati.m_Members )
				{
					if ( ti == null )
						continue;
					ti.m_InternalID = tcount++;
					byte mcount = 0;
					foreach ( BattleMemberInfo bmi in ti.m_Members )
					{
						if ( bmi == null )
							continue;
						bmi.m_InternalID = mcount++;
						if ( bmi.m_currUser )
						{
							if ( bmi.m_InternalID == 0 )
							{
								bmi.m_CBAllyTeam.Enabled = true;
								m_menuOptions.Enabled = true;
							}
							else
							{
								bmi.m_CBAllyTeam.Enabled = false;
								m_menuOptions.Enabled = false;
							}
						}
					}
				}
			}

			IDictionaryEnumerator ide = m_Members.GetEnumerator();
			TeamInfo selfTeam = m_Teams[(byte)m_Self.m_TeamID];
			AllyTeamInfo selfAT = m_AllyTeams[selfTeam.m_AllyTeamID];
			while ( ide.MoveNext() )
			{
				BattleMemberInfo bmi = (BattleMemberInfo)ide.Value;
				if ( bmi.m_currUser || bmi.m_bSpectator )
				{
					continue;
				}
				
				TeamInfo ti = m_Teams[(byte)bmi.m_TeamID];
				AllyTeamInfo ati = m_AllyTeams[ti.m_AllyTeamID];
				if ( ti.m_AllyTeamID == selfTeam.m_AllyTeamID )//||  bmi.m_InternalID != 0 || ti.m_InternalID != 0 )
				{
					if ( bmi.m_allyState != null )
					{
						m_lv.RemoveEmbeddedControl(bmi.m_allyState);
						bmi.m_allyState.Dispose();
						bmi.m_allyState = null;
					}
					continue;
					/*
					bmi.m_allyState.Visible = false;
					bmi.m_allyState.Enabled = false;
					*/
				}
				else if ( bmi.m_allyState == null )
				{
					bmi.m_allyState = new AllyState();
					bmi.m_allyState.Click += new EventHandler(m_allyState_Click);
					m_lv.AddEmbeddedControl(bmi.m_allyState, 3, m_lv.Items.IndexOf(bmi.m_lvi), DockStyle.Fill);
					bmi.m_allyState.Invalidate();
				}

				bool state = false;
				foreach ( byte members in ati.m_AlliedTo )
				{
					if ( members == selfTeam.m_AllyTeamID )
					{
						bmi.m_allyState.AlliedFrom(true);
						state = true;
					}
				}
				if ( !state )
					bmi.m_allyState.AlliedFrom(false);
				state = false;
				foreach ( byte members in selfAT.m_AlliedTo )
				{
					if ( members == ti.m_AllyTeamID )
					{
						bmi.m_allyState.AlliedTo(true);
						state = true;
					}
				}
				if ( !state )
					bmi.m_allyState.AlliedTo(false);
				/*
				if ( ti.m_AllyTeamID != selfTeam.m_AllyTeamID && bmi.m_InternalID == 0 && ti.m_InternalID == 0 )
				{
					bmi.m_allyState.Visible = true;
					bmi.m_allyState.Enabled = true;
				}
				*/
			}
		}

		public void TeamChangeAllyTeam( byte teamid, byte toallyteam )
		{
	//		lock ( this )
			{
				byte oldid = m_Teams[teamid].m_AllyTeamID;
				if ( oldid != 0xFF )
					m_AllyTeams[oldid].m_Members.RemoveAt(m_Teams[teamid].m_InternalID);
				foreach ( TeamInfo ti in m_Teams )
				{
					if ( ti.m_AllyTeamID == oldid &&
						ti.m_InternalID > m_Teams[teamid].m_InternalID )
						ti.m_InternalID--;
				}
				m_Teams[teamid].m_AllyTeamID = toallyteam;
				m_Teams[teamid].m_InternalID = (byte)m_AllyTeams[toallyteam].m_Members.Count;
				m_AllyTeams[toallyteam].m_Members.Add(m_Teams[teamid]);

				foreach ( BattleMemberInfo bmi in m_Teams[teamid].m_Members )
				{
					// if the person is the group leader and the current user, then they made the change
					// we don't want an infinite loop
					if ( bmi.m_InternalID != 0 || !bmi.m_currUser )
						bmi.m_CBAllyTeam.SelectedIndex = toallyteam;
				}

				// now cleanup the allyteam if it now has no members
				if ( oldid != 0xFF && m_AllyTeams[oldid].m_Members.Count == 0 )
				{
					m_AllyTeams[oldid].m_AlliedTo.Clear();
					foreach ( AllyTeamInfo ati in m_AllyTeams )
					{
						if ( ati.m_AlliedTo.Contains(oldid) )
							ati.m_AlliedTo.Remove(oldid);
					}
				}
				UpdateAllyStates();
			}
			DebugOut("TeamChangeAllyTeam", teamid, toallyteam);
		}

		public void AddAlliance( byte fromally, byte toally )
		{
			if ( fromally == toally )
				return;
			if ( fromally < 0 || fromally >= m_AllyTeams.Length ||
				toally < 0 || toally >= m_AllyTeams.Length )
				return;
			foreach( byte id in m_AllyTeams[fromally].m_AlliedTo )
			{
				// do nothing if they are already allied
				if ( id == toally )
					return;
			}

			m_AllyTeams[fromally].m_AlliedTo.Add(toally);
			/*
			m_AllyTeams[fromally].m_AlliedTo.Add(toally);
			TeamInfo selfteam = m_Teams[(byte)m_Self.m_TeamID];
			if ( toally == selfteam.m_AllyTeamID )
			{
				foreach ( TeamInfo ti in m_AllyTeams[fromally].m_Members )
				{
					foreach ( BattleMemberInfo bmi in ti.m_Members )
					{	
						if ( bmi.m_allyState != null )
							bmi.m_allyState.AlliedFrom(true);
					}
				}
			}
			else if ( fromally == selfteam.m_AllyTeamID )
			{
				foreach ( TeamInfo ti in m_AllyTeams[toally].m_Members )
				{
					foreach ( BattleMemberInfo bmi in ti.m_Members )
					{
//						BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id2];
						if ( bmi.m_allyState != null )
							bmi.m_allyState.AlliedTo(true);
					}
				}
			}
			*/
			UpdateAllyStates();
		}

		public void RemoveAlliance( byte fromally, byte toally )
		{
			for ( int i = 0; i < m_AllyTeams[fromally].m_AlliedTo.Count; i++ )
			{
				if ( (byte)m_AllyTeams[fromally].m_AlliedTo[i] == toally )
				{
					m_AllyTeams[fromally].m_AlliedTo.RemoveAt(i);
					UpdateAllyStates();
					return;
				}
			}
		}

		public void RemoveMember( int id )
		{
	//		lock ( this )
			{
				if ( !m_Members.Contains(id) )
					return;
				BattleMemberInfo bmi = (BattleMemberInfo)m_Members[id];
				if ( !bmi.m_bSpectator)
				{
					byte team = (byte)bmi.m_TeamID;

					m_Teams[team].m_Members.RemoveAt(bmi.m_InternalID);

					foreach ( BattleMemberInfo bmi2 in m_Teams[team].m_Members )
					{
						if ( bmi2.m_InternalID > bmi.m_InternalID )
							bmi2.m_InternalID--;
					}

					// do cleanup on team and ally team if this member was the only one in the team
					if ( m_Teams[team].m_Members.Count == 0 )
					{
						// cleanup
						byte allyteam = m_Teams[team].m_AllyTeamID;
						m_AllyTeams[allyteam].m_Members.Remove(team);
						m_Teams[team].m_InternalID = 0;
						m_Teams[team].m_AllyTeamID = 0xFF;
						m_Teams[team].m_Side = SIDE.ARM;
						if ( m_AllyTeams[allyteam].m_Members.Count == 0 )
						{
							// then cleanup this too
							m_AllyTeams[allyteam].m_AlliedTo.Clear();
							foreach ( AllyTeamInfo ati in m_AllyTeams )
							{
								if ( ati.m_AlliedTo.Contains(allyteam) )
									ati.m_AlliedTo.Remove(allyteam);
							}
						}
					}
				}

				for ( int i = 0; i < m_OrderedMembers.Count; i++ )
				{
					BattleMemberInfo bmi2 = (BattleMemberInfo)m_OrderedMembers[i];
					if ( bmi2.m_ID == id )
					{
						m_OrderedMembers.RemoveAt(i);
						break;
					}
				}
				m_lv.RemoveRow(m_lv.Items.IndexOf(bmi.m_lvi));
				m_lv.Items.Remove(bmi.m_lvi);
				m_Members.Remove(id);
				UpdateAllyStates();
			}
			CheckReady();
		}

		private void checkReady_CheckedChanged(object sender, EventArgs e)
		{
			try
			{
				CheckBox cm = (CheckBox)sender;
				if ( cm.Checked )
				{
					if ( m_Start != null )
					{
						IDictionaryEnumerator ide = m_Members.GetEnumerator();
						bool alltrue = true;
						while ( ide.MoveNext() )
						{
							if ( ((BattleMemberInfo)ide.Value).m_bSpectator )
								continue;
							if ( !((BattleMemberInfo)ide.Value).m_checkReady.Checked)
							{
								alltrue = false;
								break;
							}
						}
						if ( alltrue )
						{
							m_Start.Enabled = true;
						}
					}
				}
				else
				{
					if ( m_Start != null )
					{
						m_Start.Enabled = false;
					}
				}
				Global.battleRoom.ChangeReady(true, Global.battleRoom.GetID(), cm.Checked);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void CheckReady()
		{
			if ( m_Start != null )
			{
				IDictionaryEnumerator ide = m_Members.GetEnumerator();
				bool alltrue = true;
				while ( ide.MoveNext() )
				{
					if ( ((BattleMemberInfo)ide.Value).m_bSpectator )
						continue;
					if ( !((BattleMemberInfo)ide.Value).m_checkReady.Checked)
					{
						alltrue = false;
						break;
					}
				}
				if ( alltrue )
				{
					m_Start.Enabled = true;
				}
			}
		}

		// this isn't quite right yet.  get back to this
		private void m_allyState_Click(object sender, EventArgs e)
		{
			AllyState state = (AllyState)sender;
			if ( m_Debug )
			{
				m_DebugWriter.WriteLine("Attempting to change ally state");
			}
			// here is where we send an ally clicked message
			IDictionaryEnumerator ide = m_Members.GetEnumerator();
			while ( ide.MoveNext() )
			{
				BattleMemberInfo bmi = (BattleMemberInfo)ide.Value;
				if ( object.Equals(bmi.m_allyState, sender) )
				{
					if ( bmi.m_InternalID == 0 )
					{
						TeamInfo tilhs = m_Teams[(int)m_Self.m_TeamID];
						TeamInfo tirhs = m_Teams[(int)bmi.m_TeamID];
						if ( m_Debug )
						{
							m_DebugWriter.WriteLine("Actually changing ally state");
						}
						if ( (state.State & AllyState.STATE.LEFT) != 0 )
							Global.battleRoom.SendRemoveAllyMessage(tilhs.m_AllyTeamID, tirhs.m_AllyTeamID);
						else
							Global.battleRoom.SendAllyMessage(tilhs.m_AllyTeamID, tirhs.m_AllyTeamID);
					}
					return;
				}
			}
		}
	}
}